package com.install4j.api.beans;

import com.install4j.api.beaninfo.ScriptParameter;
import com.install4j.api.context.Context;
import com.install4j.api.context.InstallerContext;
import com.install4j.api.context.UserCanceledException;
import com.install4j.runtime.installer.ContextImpl;
import com.install4j.runtime.installer.InstallerVariables;

import java.io.File;

/**
 * Common base class for all abstract super-classes for beans that are handled by install4j.
 * It is not recommended to extend this class directly. Concrete super-classes exist for
 * <ul>
 * <li><a href="../screens/package-summary.html">screens</a></li>
 * <li><a href="../actions/package-summary.html">actions</a></li>
 * <li><a href="../formcomponents/package-summary.html">form components</a></li>
 * </ul>
 * <p>This class provides common utility methods fir all bean types.
 * @author ej-technologies GmbH
 */
public abstract class AbstractBean implements Bean {

    /**
     * Same as {@link #replaceVariables(String, ReplacementMode, VariableErrorHandlingDescriptor)}, with
     * {@link ReplacementMode#PLAIN} as the replacement mode and
     * {@link VariableErrorHandlingDescriptor#DEFAULT} as the error handling descriptor.
     * @param value the original string
     * @return the string with all variables replaced.
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String replaceVariables(String value) throws UndefinedVariableException {
        return InstallerVariables.replaceVariables(value);
    }

    /**
     * Same as {@link #replaceVariables(String, ReplacementMode, VariableErrorHandlingDescriptor)}, with
     * {@link VariableErrorHandlingDescriptor#DEFAULT} as the error handling descriptor.
     * @param value the original string
     * @param replacementMode the replacement mode
     * @return the string with all variables replaced.
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String replaceVariables(String value, ReplacementMode replacementMode) throws UndefinedVariableException {
        return InstallerVariables.replaceVariables(value, replacementMode);
    }

    /**
     * Same as {@link #replaceVariables(String, ReplacementMode, VariableErrorHandlingDescriptor)}, with
     * {@link ReplacementMode#PLAIN} as the replacement mode.
     * @param value the original string
     * @param errorHandlingDescriptor describes how missing variables should be treated for each variable type
     * @return the string with all variables replaced.
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String replaceVariables(String value, VariableErrorHandlingDescriptor errorHandlingDescriptor) throws UndefinedVariableException  {
        return InstallerVariables.replaceVariables(value, ReplacementMode.PLAIN, errorHandlingDescriptor);
    }

    /**
     * Replace all installer variables and localization keys in a string.
     * <p>Note: compiler variables in the project are replaced at compile time. Compiler variables in external files,
     * such as a custom localization file are not replaced at a compile time.
     * @param value the original string
     * @param replacementMode the replacement mode
     * @param errorHandlingDescriptor describes how missing variables should be treated for each variable type
     * @return the string with all variables replaced.
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String replaceVariables(String value, ReplacementMode replacementMode, VariableErrorHandlingDescriptor errorHandlingDescriptor) throws UndefinedVariableException {
        return InstallerVariables.replaceVariables(value, replacementMode, errorHandlingDescriptor);
    }

    /**
     * Replace all installer variables and localization keys in a file name.
     * The used error handling descriptor is {@link VariableErrorHandlingDescriptor#DEFAULT}.
     * <p>Note: compiler variables in the project are replaced at compile time. Compiler variables in external files,
     * such as a custom localization file are not replaced at a compile time.
     * @param file the original file
     * @return the file with all variables replaced in its name.
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static File replaceVariables(File file) throws UndefinedVariableException {
        return InstallerVariables.replaceVariables(file);
    }

    /**
     * Same as {@link #replaceVariables(String[], VariableErrorHandlingDescriptor)}, with
     * {@link VariableErrorHandlingDescriptor#DEFAULT} as the error handling descriptor.
     * @param values the original array
     * @return the array with all variables replaced
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String[] replaceVariables(String[] values) throws UndefinedVariableException {
        return replaceVariables(values, VariableErrorHandlingDescriptor.DEFAULT);
    }

    /**
     * Replace all installer variables and localization keys in a string array.
     * For array elements that consist of an installer variable that in turn contains
     * an array value, the elements of that array value will be inserted into
     * the returned array. For this reason the returned array may have more elements than the
     * original array. For example, if the array passed in as an argument has the elements:
     * <pre>{@code
     *   [0] = "One"
     *   [1] = "${installer:myVariable}"
     *   [2] = "Three"
     * }</pre>
     * and the variable {@code myVariable} has a String array value with the elements
     * <pre>{@code
     *   [0] = "Blue"
     *   [1] = "Green"
     * }</pre>
     * then the returned array will be
     * <pre>{@code
     *   [0] = "One"
     *   [1] = "Blue"
     *   [2] = "Green"
     *   [3] = "Three"
     * }</pre>
     *<p>
     * If the array in an installer variable is not of type {@code String[]},
     * each element will be converted to a string by calling {@code toString()} on it.
     * </p>
     * <p>Note: compiler variables are replaced at compile time.
     * @param values the original array
     * @param errorHandlingDescriptor describes how missing variables should be treated for each variable type
     * @return the array with all variables replaced
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static String[] replaceVariables(String[] values, VariableErrorHandlingDescriptor errorHandlingDescriptor) throws UndefinedVariableException {
        return InstallerVariables.replaceVariables(values, errorHandlingDescriptor);
    }

    /**
     * Replace all installer variables and localization keys in a file array.
     * This method is analogous to {@link #replaceVariables(String[])}.
     * @param files the original array
     * @return the array with all variables replaced
     * @throws UndefinedVariableException if a variable name cannot be found and the error handling is set to
     * {@link VariableErrorHandling#EXCEPTION} for the variable type.
     */
    public static File[] replaceVariables(File[] files) throws UndefinedVariableException {
        if (files == null) {
            return null;
        } else {
            String[] replacedFileNames = InstallerVariables.replaceVariables(files, VariableErrorHandlingDescriptor.DEFAULT);
            File[] replacedFiles = new File[replacedFileNames.length];
            for (int i = 0; i < replacedFileNames.length; i++) {
                replacedFiles[i] = new File(replacedFileNames[i]);
            }
            return replacedFiles;
        }
    }

    /**
     * Synchronously execute an action list.
     * You should not use this method from a form component, because it will block the EDT. Use
     * {@link #executeActionListAsync(ActionList, Object...)} instead.
     * @param actionList the action list object from your bean property.
     * @param extraScriptParameters if the action list provides extra script parameters in the BeanInfo by calling
     *          {@link com.install4j.api.beaninfo.ActionListPropertyDescriptor#setExtraScriptParameters(ScriptParameter[])},
     *                        you have to pass them here.
     * @return whether the list of actions was terminated due to an error. This can only happen if the user selected
     *         the "Break if an error occurs" check box in the configuration dialog of the action list, i.e.
     *         if {@link ActionList#isBreakOnError()} of the argument returns {@code true}.
     *         To detect errors in absence of this flag, call {@link Context#setErrorOccurred(boolean)}
     *         with an argument of {@code false} before calling this method and check the result of
     *         {@link Context#isErrorOccurred()} afterwards.
     */
    public static boolean executeActionListSync(ActionList actionList, Object... extraScriptParameters) throws UserCanceledException {
        return ContextImpl.getSingleContextInt().executeActionListSync(actionList, extraScriptParameters);
    }

    /**
     * Asynchronously execute an action list.
     * Do not invoke this methods from an action, it will throw an {@code IllegalStateException} in that case. For
     * actions, use {@link #executeActionListSync(ActionList, Object...)} instead.
     * This method is intended to be used form a form component and will disable all components on the current screen
     * while the action list is executing.
     * @param actionList the action list object from your bean property.
     * @param extraScriptParameters if the action list provides extra script parameters in the BeanInfo by calling
     *          {@link com.install4j.api.beaninfo.ActionListPropertyDescriptor#setExtraScriptParameters(ScriptParameter[])},
     *                        you have to pass them here.
     */
    public static void executeActionListAsync(ActionList actionList, Object... extraScriptParameters) {
        ContextImpl.getSingleContextInt().executeActionListAsync(actionList, extraScriptParameters);
    }

    /**
     * Roll back an action list.
     * This will only have an effect if called from the {@link com.install4j.api.actions.InstallAction#rollback(InstallerContext)}
     * method and if you called {@link #executeActionListSync(ActionList, Object...)} before
     * @param actionList the action list object from your bean property
     */
    public static void rollbackActionList(ActionList actionList) {
        ContextImpl.getSingleContextInt().rollbackActionList(actionList);
    }
}
